@@ -14,9 +14,12 @@ from pathlib import Path, PureWindowsPath
 from typing import Union, List, Tuple
 import re
 import datetime
+import signal
 
 import numpy as np
 import pandas as pd
+from scipy.sparse import csr_matrix
+import networkx as nx
 import pythoncom
 import win32com
 from win32com.client import VARIANT
@@ -580,6 +583,192 @@ class SAW(object):
             else:
                 raise e from None
 
+    def get_ybus(self, full: bool = False, file: Union[str, None] = None) -> \
+            Union[np.ndarray, csr_matrix]:
+        """Helper to obtain the YBus matrix from PowerWorld (in Matlab sparse
+        matrix format) and then convert to scipy csr_matrix by default.
+        :param full: Convert the csr_matrix to the numpy array (full matrix).
+        :param file: Path to the external Ybus file.
+        """
+        if file:
+            _tempfile_path = file
+        else:
+            _tempfile = tempfile.NamedTemporaryFile(mode='w', suffix='.mat',
+                                                    delete=False)
+            _tempfile_path = Path(_tempfile.name).as_posix()
+            _tempfile.close()
+            cmd = 'SaveYbusInMatlabFormat("{}", NO)'.format(_tempfile_path)
+            self.RunScriptCommand(cmd)
+        with open(_tempfile_path, 'r') as f:
+            f.readline()
+            mat_str = f.read()
+        mat_str = re.sub(r'\s', '', mat_str)
+        lines = re.split(';', mat_str)
+        ie = r'[0-9]+'
+        fe = r'-*[0-9]+\.[0-9]+'
+        dr = re.compile(r'(?:Ybus)=(?:sparse\()({ie})'.format(ie=ie))
+        exp = re.compile(
+            r'(?:Ybus\()({ie}),({ie})(?:\)=)({fe})(?:\+j\*)(?:\()({fe})'.format(
+                ie=ie, fe=fe))
+        # Get the dimension from the first line in lines
+        dim = dr.match(lines[0]).group(1)
+        n = int(dim)
+        row = []
+        col = []
+        data = []
+        for line in lines[1:]:
+            match = exp.match(line)
+            if match is None:
+                continue
+            idx1, idx2, real, imag = match.groups()
+            # Type conversion can be optimized to provide slightly
+            # improvement
+            admittance = float(real) + 1j * float(imag)
+            row.append(int(idx1))
+            col.append(int(idx2))
+            data.append(admittance)
+        # The row index is always in the ascending order in the mat file
+        sparse_matrix = csr_matrix(
+            (data, (np.asarray(row) - 1, np.asarray(col) - 1)), shape=(n, n),
+            dtype=complex)
+        if full:
+            return sparse_matrix.toarray()
+        else:
+            return sparse_matrix
+
+    def get_jacobian(self, full=False):
+        """Helper function to get the Jacobian matrix, by default return a
+        scipy sparse matrix in the csr format
+        :param full: Convert the csr_matrix to the numpy array (full matrix).
+        """
+        jacfile = tempfile.NamedTemporaryFile(mode='w', suffix='.m',
+                                              delete=False)
+        jacfile_path = Path(jacfile.name).as_posix()
+        jacfile.close()
+        jidfile = tempfile.NamedTemporaryFile(mode='w', delete=False)
+        jidfile_path = Path(jidfile.name).as_posix()
+        jidfile.close()
+        cmd = 'SaveJacobian("{}","{}",M,R);'.format(jacfile_path, jidfile_path)
+        self.RunScriptCommand(cmd)
+        with open(jacfile_path, 'r') as f:
+            mat_str = f.read()
+        os.unlink(jacfile.name)
+        os.unlink(jidfile.name)
+        mat_str = re.sub(r'\s', '', mat_str)
+        lines = re.split(';', mat_str)
+        ie = r'[0-9]+'
+        fe = r'-*[0-9]+\.[0-9]+'
+        dr = re.compile(r'(?:Jac)=(?:sparse\()({ie})'.format(ie=ie))
+        exp = re.compile(
+            r'(?:Jac\()({ie}),({ie})(?:\)=)({fe})'.format(
+                ie=ie, fe=fe))
+        row = []
+        col = []
+        data = []
+        # Get the dimension from the first line in lines
+        dim = dr.match(lines[0]).group(1)
+        n = int(dim)
+        for line in lines[1:]:
+            match = exp.match(line)
+            if match is None:
+                continue
+            idx1, idx2, real = match.groups()
+            row.append(int(idx1))
+            col.append(int(idx2))
+            data.append(float(real))
+        sparse_matrix = csr_matrix(
+            (data, (np.asarray(row) - 1, np.asarray(col) - 1)), shape=(n, n))
+        if full:
+            return sparse_matrix.toarray()
+        else:
+            return sparse_matrix
+
+    def to_graph(self, node: str = 'bus', geographic: bool = False,
+                 directed: bool = False, node_attr=None, edge_attr=None) \
+            -> Union[nx.MultiGraph, nx.MultiDiGraph]:
+        """Generate the graph network model (NetworkX) from the topology.
+        Currently supports the bus-level topology and the inter-substation
+        level topology. Parallel lines (if exist) are preserved in the
+        model.
+        :param edge_attr:  A valid field name(str) or iterable of
+        field names that are used to retrieve values and add them to the
+        graph as edge attributes. All fields belonging to object branch are
+        available.
+        :param node_attr: A valid field name(str) or iterable of
+        field names that are used to retrieve values and add them to the
+        graph as node attributes. All fields belonging to the node type are
+        available.
+        :param directed: Whether to convert to a directed graph (based on
+        the direction of real power flow).
+        :param geographic: Include latitude and longitude in the node's
+        attributes. If geographic information is unavailable in the case,
+        the latitude and the longitude will be NaN.
+        :param node: Elements to be represented by nodes. Only 'bus' or
+        'substation' is supported.
+        """
+        if node not in ['bus', 'substation']:
+            raise ValueError(
+                "Currently only support 'bus' or 'substation' as the node "
+                "type.")
+        qf = []
+        nf = []
+        node_from = None
+        node_to = None
+        if node == 'bus':
+            qf = self.get_key_field_list('branch') + ['LineMW']
+            node_from = "BusNum"
+            node_to = "BusNum:1"
+            nf = ['BusNum']
+            if geographic:
+                nf += ['Latitude:1', 'Longitude:1']
+        elif node == 'substation':
+            qf = self.get_key_field_list('branch') + ['SubNum', 'SubNum:1',
+                                                      'LineMW']
+            node_from = "SubNum"
+            node_to = "SubNum:1"
+            nf = ['SubNum']
+            if geographic:
+                nf += ['Latitude', 'Longitude']
+        if edge_attr:
+            if isinstance(edge_attr, (list, tuple)):
+                qf.extend(edge_attr)
+            else:
+                qf.append(edge_attr)
+                edge_attr = [edge_attr]
+        try:
+            branch_df = self.GetParametersMultipleElement('branch', qf)
+        except ValueError as e:
+            raise e
+        if directed:
+            graph_type = nx.MultiDiGraph
+            for index, row in branch_df.iterrows():
+                linemw = row['LineMW']
+                if linemw < 0:
+                    original_from = row[node_from]
+                    original_to = row[node_to]
+                    branch_df.loc[index, 'LineMW'] = abs(linemw)
+                    branch_df.loc[index, node_from] = original_to
+                    branch_df.loc[index, node_to] = original_from
+        else:
+            graph_type = nx.MultiGraph
+        graph = nx.from_pandas_edgelist(branch_df, node_from, node_to,
+                                        create_using=graph_type,
+                                        edge_key="LineCircuit",
+                                        edge_attr=edge_attr)
+        if node_attr:
+            if isinstance(node_attr, (list, tuple)):
+                nf.extend(node_attr)
+            else:
+                nf.append(node_attr)
+        if geographic or node_attr:
+            try:
+                node_df = self.GetParametersMultipleElement(node, nf)
+                node_attr_reformat = node_df.set_index(node_from).to_dict('index')
+                nx.set_node_attributes(graph, node_attr_reformat)
+            except ValueError as e:
+                raise e
+        return graph
+
     def _set_simauto_property(self, property_name, property_value):
         """Helper to just set a property name and value. Primary purpose
         of breaking things out this way is for testing.
@@ -1538,7 +1727,7 @@ class SAW(object):
     def OpenOneLine(self, filename: str, view: str = "",
                     FullScreen: str = "NO", ShowFull: str = "NO",
                     LinkMethod: str = "LABELS", Left: float = 0.0,
-                    Top: float = 0.0, Width: float = 0.0, Height: float = 0.0)\
+                    Top: float = 0.0, Width: float = 0.0, Height: float = 0.0) \
             -> None:
         """Use this function to open a oneline diagram. This function
         can be used to associate onelines with a PWB file.
